import pytest
import uuid

from conftrak.client.utils import doc_or_uid_to_uid


def test_docuid_uid():
    uid = str(uuid.uuid4())
    doc = dict(uid=uid)
    assert uid == doc_or_uid_to_uid(uid)
    assert uid == doc_or_uid_to_uid(doc)




from doct import Document
import ujson
import requests
from ..client import conf
from .utils import _get, _post, _put, _delete


class ConfigurationReference(object):
    """Reference implementation of generic configuration manager"""
    def __init__(self, host=conf.conn_config['host'],
                 port=conf.conn_config['port']):
        """Constructor

        Parameters
        ----------
        host: str, optional
            Machine name/address for ConfTrak server
        port: int, optional
            Port ConfTrak server is initiated on

        """
        self.host = host
        self.port = port

    @property
    def _server_path(self):
        """URL to the ConfTrak server"""
        return 'http://{}:{}/'.format(self.host, self.port)

    @property
    def _conf_url(self):
        """URL to the configuration reference handler in the server side"""
        return self._server_path + 'configuration'

    @property
    def _schema_url(self):
        """URL to the schema reference handler in the server side"""
        return self._server_path + 'schema'

    def create(self, beamline_id, time=None, uid=None, **kwargs):
        """Insert a configuration to the database

        Parameters
        ----------
        beamline_id : str
            The beamline identifier.
        time : float
            The time that the configuration was created.
        uid : str
            Unique identifier for a configuration document

        Returns
        -------
        ins_doc : str
            The inserted document

        """
        doc = dict(uid=uid,
                   beamline_id=beamline_id,
                   time=time,
                   **kwargs)
        ins_doc = _post(self._conf_url, doc)
        return ins_doc[0]

    def update(self, query, update):
        """Update a configuration given a query and name value pair to be
        updated. No upsert support. For more info on upsert, check Mongo
        documentations.

        Parameters
        -----------
        query: dict
            Allows finding Sample documents to be updated
        update: dict
            Name/value pair that is to be replaced within an existing Request doc

        Returns
        ----------
        bool
            Returns True if update successful

        """
        _put(self._conf_url, query, update)
        return True

    def find(self, as_document=False, active_only=True, **kwargs):
        """
        Parameters
        ----------
        as_document: bool
            Formats output to doct.Document if True
        active_only: bool
            Retrieve only active configurations. Default is True

        Yields
        ------
        c: dict, doct.Document
            Documents which have all keys with the given values

        """
        kwargs['active_only'] = active_only
        content = _get(self._conf_url, params=kwargs)
        if as_document:
            for c in content:
                yield Document('Configuration', c)
        else:
            for c in content:
                yield c

    def delete(self, uid_list):
        """Virtually delete (mark as inactive) the informed configurations.

        Parameters
        ----------
        uid_list: list
            List of unique identifiers to be marked as inactive.

        """
        _delete(self._conf_url, params={'uid_list':uid_list})
        return True

    def get_schema(self):
        """Get information about schema from server side

        Returns
        -------
        dict
            Returns the json schema dict used for validation

        """
        r = requests.get(self._schema_url,
                         params=ujson.dumps('configuration'))
        r.raise_for_status()
        return ujson.loads(r.text)




import sys
import uuid
import ujson
import pytest
import time as ttime
from mock import patch
from conftrak.ignition import Application, parse_configuration
from conftrak.server.engine import (db_connect)
from conftrak.server.utils import ConfTrakException
from .utils import testing_config as TESTING_CONFIG


@pytest.fixture
def app():
    db = db_connect(TESTING_CONFIG['database'],
                    TESTING_CONFIG['mongohost'],
                    TESTING_CONFIG['mongoport'])
    return Application(db)


def test_parse_configuration():
    testargs = ["prog", "--database", "conftrak", "--mongo-host", "localhost",
                "--mongo-port", "27017", #"--service-port", "7771",
                "--timezone", "US/Eastern"]
    with patch.object(sys, 'argv', testargs):
        config = parse_configuration(dict())
        assert config['service_port'] == 7771


def test_db_connect():
    db_connect(TESTING_CONFIG['database'],
               TESTING_CONFIG['mongohost'],
               TESTING_CONFIG['mongoport'])

    with pytest.raises(ConfTrakException):
        db_connect(TESTING_CONFIG['database'],
                   'invalid_mongo_host',
                   TESTING_CONFIG['mongoport'])


@pytest.mark.gen_test
def test_configuration_post(http_server, http_client, base_url):
    payload = dict(beamline_id='test_bl', uid=str(uuid.uuid4()),
                   active=True, time=ttime.time(), key='test_config',
                   params=dict(param1='test1', param2='test2'))
    url = '{}/{}'.format(base_url, 'configuration')
    body = ujson.dumps(payload)
    response = yield http_client.fetch(url, method='POST', body=body,
                                       raise_error=False)
    assert response.code == 200

    # Test submiting a list
    payload = [dict(beamline_id='test_bl', uid=str(uuid.uuid4()),
                    active=True, time=ttime.time(), key='test_config',
                    params=dict(param1='test1', param2='test2'))]
    url = '{}/{}'.format(base_url, 'configuration')
    body = ujson.dumps(payload)
    response = yield http_client.fetch(url, method='POST', body=body,
                                       raise_error=False)
    assert response.code == 200

    # Test invalid schema
    payload = dict(a=1, b=2)
    url = '{}/{}'.format(base_url, 'configuration')
    body = ujson.dumps(payload)
    response = yield http_client.fetch(url, method='POST', body=body,
                                       raise_error=False)
    assert response.code == 400

    payload = [dict(a=1, b=2)]
    url = '{}/{}'.format(base_url, 'configuration')
    body = ujson.dumps(payload)
    response = yield http_client.fetch(url, method='POST', body=body,
                                       raise_error=False)
    assert response.code == 400

    # Test invalid payload format. The handler expects list or dict
    payload = 1
    url = '{}/{}'.format(base_url, 'configuration')
    body = ujson.dumps(payload)
    response = yield http_client.fetch(url, method='POST', body=body,
                                       raise_error=False)
    assert response.code == 500


@pytest.mark.gen_test
def test_configuration_get(http_server, http_client, base_url):
    # Insert the test data
    payload_insert = dict(beamline_id='test_bl', uid=str(uuid.uuid4()),
                          active=True, time=ttime.time(), key='test_config',
                          params=dict(param1='test1', param2='test2'))
    url = '{}/{}'.format(base_url, 'configuration')
    body = ujson.dumps(payload_insert)
    response = yield http_client.fetch(url, method='POST', body=body,
                                       raise_error=False)

    url = '{}/{}'.format(base_url, 'configuration')
    body = None

    # Test valid query
    payload = ujson.dumps(dict(active_only=True))
    g_url = url + "?" + payload
    response = yield http_client.fetch(g_url, method='GET', body=body,
                                       raise_error=False)
    assert response.code == 200

    # Test valid query with limit on number of rows
    payload = ujson.dumps(dict(active_only=False, num=1))
    g_url = url + "?" + payload
    response = yield http_client.fetch(g_url, method='GET', body=body,
                                       raise_error=False)
    assert response.code == 200

    # Test valid query without results on DB 
    payload = ujson.dumps(dict(active_only=False, uid="-1"))
    g_url = url + "?" + payload
    response = yield http_client.fetch(g_url, method='GET', body=body,
                                       raise_error=False)
    assert response.code == 500

    # Test invalid query
    payload = ujson.dumps({"$uid":"{'non_existent':1}", "num":1})
    g_url = url + "?" + payload
    response = yield http_client.fetch(g_url, method='GET', body=body,
                                       raise_error=False)
    print("Response: ", response)
    assert response.code == 500


@pytest.mark.gen_test
def test_configuration_put(http_server, http_client, base_url):
    # Insert the test data
    payload_insert = dict(beamline_id='test_bl', uid=str(uuid.uuid4()),
                          active=True, time=ttime.time(), key='test_config',
                          params=dict(param1='test1', param2='test2'))
    url = '{}/{}'.format(base_url, 'configuration')
    body = ujson.dumps(payload_insert)
    response = yield http_client.fetch(url, method='POST', body=body,
                                       raise_error=False)

    # Update the key field
    payload_update = dict(query={'uid': payload_insert['uid']},
                          update={'key': 'updated_key'})
    body = ujson.dumps(payload_update)
    response = yield http_client.fetch(url, method='PUT', body=body,
                                       raise_error=False)

    assert response.code == 200

    # Test invalid arguments. Query and Update are required
    payload_update = dict(update={'key': 'updated_key'})
    body = ujson.dumps(payload_update)
    response = yield http_client.fetch(url, method='PUT', body=body,
                                       raise_error=False)

    assert response.code == 500

    # Test invalid update. Updates on UID and Time are not allowed
    payload_update = dict(query={'uid': payload_insert['uid']},
                          update={'uid': 'this_must_be_invalid'})
    body = ujson.dumps(payload_update)
    response = yield http_client.fetch(url, method='PUT', body=body,
                                       raise_error=False)

    assert response.code == 500

    payload_update = dict(query={'uid': payload_insert['uid']},
                          update={'time': ttime.time()})
    body = ujson.dumps(payload_update)
    response = yield http_client.fetch(url, method='PUT', body=body,
                                       raise_error=False)

    assert response.code == 500


@pytest.mark.gen_test
def test_configuration_delete(http_server, http_client, base_url):
    payload_insert = dict(beamline_id='test_bl', uid=str(uuid.uuid4()),
                          active=True, time=ttime.time(), key='test_config',
                          params=dict(param1='test1', param2='test2'))
    url = '{}/{}'.format(base_url, 'configuration')
    body = ujson.dumps(payload_insert)
    response = yield http_client.fetch(url, method='POST', body=body,
                                       raise_error=False)

    # Test a valid delete
    payload_delete = dict(uid_list=[payload_insert['uid']])
    d_url = url+"?"+ujson.dumps(payload_delete)
    body = None
    response = yield http_client.fetch(d_url, method='DELETE', body=body,
                                       raise_error=False)

    assert response.code == 200

    # Test delete sending a single uid, not a list
    payload_delete = dict(uid_list=payload_insert['uid'])
    d_url = url+"?"+ujson.dumps(payload_delete)
    body = None
    response = yield http_client.fetch(d_url, method='DELETE', body=body,
                                       raise_error=False)

    assert response.code == 200

    # Test delete must fail if a list of uids is not given.
    payload_delete = dict(my_param=[payload_insert['uid']])
    d_url = url+"?"+ujson.dumps(payload_delete)
    body = None
    response = yield http_client.fetch(d_url, method='DELETE', body=body,
                                       raise_error=False)

    assert response.code == 500


@pytest.mark.gen_test
def test_schema_get(http_server, http_client, base_url):
    url = '{}/{}'.format(base_url, 'schema')
    body = None
    url = url + '?'+ujson.dumps('configuration')
    response = yield http_client.fetch(url, method='GET', body=body,
                                       raise_error=False)
    assert response.code == 200


@pytest.mark.gen_test
def test_schema_put(http_server, http_client, base_url):
    url = '{}/{}'.format(base_url, 'schema')
    body = 'configuration'
    response = yield http_client.fetch(url, method='PUT', body=body,
                                       raise_error=False)
    assert response.code == 405


@pytest.mark.gen_test
def test_schema_post(http_server, http_client, base_url):
    url = '{}/{}'.format(base_url, 'schema')
    body = 'configuration'
    response = yield http_client.fetch(url, method='POST', body=body,
                                       raise_error=False)
    assert response.code == 405



from doct import Document
import time as ttime
import pytest
from .utils import conftrak_setup, conftrak_teardown
from .utils import testing_config as TESTING_CONFIG
from conftrak.client.commands import ConfigurationReference
from conftrak.exceptions import ConfTrakNotFoundException
from requests.exceptions import HTTPError, RequestException

import uuid
import ujson
import jsonschema

configs_uids = []
document_insertion_times = []


@pytest.fixture(scope='function')
def config_ref():
    c = ConfigurationReference()
    c.host = TESTING_CONFIG['mongohost']
    c.port = TESTING_CONFIG['serviceport']
    return c


def setup_module(module):
    """ setup any state specific to the execution of the given module."""
    conftrak_setup()


def teardown_module(module):
    """ teardown any state that was previously setup with a setup_module
    method.
    """
    conftrak_teardown()


def test_commands_smoke():
    c_create = ConfigurationReference.create
    c_find = ConfigurationReference.find
    c_update = ConfigurationReference.update
    c_delete = ConfigurationReference.delete
    c_get_schema = ConfigurationReference.get_schema


def test_configuration_constructor():
    c2 = ConfigurationReference()


def test_connection_switch():
    c = ConfigurationReference()
    c.host = 'blah'
    pytest.raises(RequestException, c.create, 'test_beamline')
    c.host = TESTING_CONFIG['mongohost']
    c.create(beamline_id='lix')


def test_configuration_create(config_ref):
    c1 = config_ref.create(beamline_id='test')
    c2 = config_ref.create(beamline_id='test', uid=str(uuid.uuid4()))
    c_kwargs = dict(key='detector', params={'model':'Pilatus1M',
                                            'vendor':'Dectris'})
    c3 = config_ref.create(beamline_id='test', **c_kwargs)


def test_configuration_find(config_ref):
    config_data = dict(beamline_id='test_bl', uid=str(uuid.uuid4()),
                       active=True, time= ttime.time(),key='test_config',
                       params=dict(param1='test1', param2='test2'))

    config_ref.create(**config_data)
    c_ret = next(config_ref.find(uid=config_data['uid'], as_document=True))
    assert c_ret == Document('Configuration', config_data)


def test_configuration_update(config_ref):
    config_data = dict(beamline_id='test_bl', uid=str(uuid.uuid4()),
                       active=True, time= ttime.time(),key='test_config',
                       params=dict(param1='test1', param2='test2'))
    config_ref.create(**config_data)
    config_ref.update(query={'uid': config_data['uid']},
                      update={'key': 'updated_key'})
    updated_conf = next(config_ref.find(uid=config_data['uid']))
    assert updated_conf['key'] == 'updated_key'


def test_configuration_delete(config_ref):
    config_data = dict(beamline_id='test_bl', uid=str(uuid.uuid4()),
                       active=True, time= ttime.time(),key='test_config',
                       params=dict(param1='test1', param2='test2'))
    config_ref.create(**config_data)
    inserted = next(config_ref.find(uid=config_data['uid']))
    # Make sure that it was inserted
    assert inserted is not None
    config_ref.delete([config_data['uid']])
    with pytest.raises(ConfTrakNotFoundException):
        deleted = next(config_ref.find(uid=config_data['uid']))


def test_configuration_find_all(config_ref):
    config_data = dict(beamline_id='test_bl', uid=str(uuid.uuid4()),
                       active=True, time= ttime.time(),key='test_config',
                       params=dict(param1='test1', param2='test2'))
    config_ref.create(**config_data)
    inserted = next(config_ref.find(uid=config_data['uid']))
    # Make sure that it was inserted
    assert inserted is not None
    config_ref.delete([config_data['uid']])
    deleted = next(config_ref.find(active_only=False, uid=config_data['uid']))
    assert deleted is not None


def test_configuration_schema(config_ref):
    config_data = dict(beamline_id='test_bl', uid=str(uuid.uuid4()),
                       active=True, time=ttime.time(),key='test_config',
                       params=dict(param1='test1', param2='test2'))
    schema = config_ref.get_schema()
    try:
        jsonschema.validate(config_data, schema)
    except:
        pytest.fail("test_configuration_schema failed on validate schema.")




from __future__ import absolute_import
from tornado import gen
import tornado.web
import pymongo
import jsonschema
import ujson
from . import utils
from jsonschema.exceptions import ValidationError, SchemaError
from ..exceptions import ConfTrakException


def db_connect(database, mongo_host, mongo_port):
    """Helper function to deal with stateful connections to MongoDB
    Connection established lazily. Connects to the database on request.
    Same connection pool is used for all clients per recommended by
    tornado developer manual.
    Parameters
    ----------
    database: str
        The name of database pymongo creates and/or connects
    host: str
        Name/address of the server that mongo daemon lives
    port: int
        Port num of the server
    Returns pymongo.database.Database
    -------
        Async server object which comes in handy as server has to juggle
    multiple clients and makes no difference for a single client compared
    to pymongo
    """
    try:
        client = pymongo.MongoClient(host=mongo_host, port=mongo_port)
        client.list_database_names()  # check if the server is really okay.
    except (pymongo.errors.ConnectionFailure,
            pymongo.errors.ServerSelectionTimeoutError):
        raise ConfTrakException("Unable to connect to MongoDB server...")

    database = client[database]

    database.configuration.create_index([('uid', pymongo.DESCENDING)],
                                        unique=True, background=False)
    database.configuration.create_index([('time', pymongo.DESCENDING)],
                                        unique=False, background=True)
    database.configuration.create_index([('beamline_id', pymongo.DESCENDING)],
                                        unique=False, background=True,
                                        sparse=True)
    database.configuration.create_index([('key', pymongo.DESCENDING)],
                                        unique=False, background=True,
                                        sparse=True)

    return database


class DefaultHandler(tornado.web.RequestHandler):
    """DefaultHandler which takes care of CORS. Javascript needs this.
    In general, methods on RequestHandler and elsewhere in Tornado
    are not thread-safe. In particular, methods such as write(),
    finish(), and flush() must only be called from the main thread.
    If you use multiple threads it is important to use IOLoop.add_callback
    to transfer control back to the main thread before finishing the request.
    """
    @gen.coroutine
    def set_default_headers(self):
        self.set_header('Access-Control-Allow-Origin', '*')
        self.set_header('Access-Control-Allow-Methods', 'POST, GET, OPTIONS')
        self.set_header('Access-Control-Max-Age', 1000)
        self.set_header('Access-Control-Allow-Headers', '*')
        self.set_header('Content-type', 'application/json')

    def data_received(self, chunk):
        """Abstract method, overwrite potential default"""
        pass


class ConfigurationReferenceHandler(DefaultHandler):
    """Handler for ConfigurationReference insert, update, and querying.
    A RESTful handler, nothing fancy or stateful about this
    Methods
    --------
    get()
        Query 'configuration_referenece' documents given certain parameters.
    Pass 'num' in order to obtain the last 'num' configuration references.
    post()
        Insert 'configuration' documents
    put()
        Update or Insert if 'configuration' document does not exist.
    Update supports both field update and document replacement. If you
    would like to replace a document, simply provide a full doc in update
    field. Otherwise, provide a dict that holds the new value and field name.
    Returns the total number of documents that are updated.
    """
    @gen.coroutine
    def get(self):
        database = self.settings['db']
        query = utils.unpack_params(self)
        if 'active_only' in query:
            filter_active = query.pop('active_only')
            if filter_active:
                query['active'] = True

        num = query.pop("num", None)
        try:
            if num:
                docs = database.configuration.find(query).sort('time',
                                                               direction=pymongo.DESCENDING).limit(num)
            else:
                docs = database.configuration.find(query).sort('time',
                                                               direction=pymongo.DESCENDING)
            num_docs = database.configuration.count_documents(query)
            if docs and num_docs > 0:
                utils.return2client(self, docs)
            else:
                raise utils._compose_err_msg(500, 'No results found!')
        except pymongo.errors.PyMongoError:
            raise utils._compose_err_msg(500, 'Query on config has failed',
                                         query)


    @gen.coroutine
    def post(self):
        database = self.settings['db']
        data = ujson.loads(self.request.body.decode("utf-8"))
        uids = []
        if isinstance(data, list):
            for d in data:
                # Ensure the active status on the new Configuration
                d['active'] = True
                d = utils.default_timeuid(d)
                try:
                    jsonschema.validate(d,
                                        utils.schemas['configuration'])
                except (ValidationError, SchemaError):
                    raise utils._compose_err_msg(400,
                                                 "Invalid schema on document(s)", d)
                uids.append(d['uid'])
                database.configuration.insert_one(d)
        elif isinstance(data, dict):
            data = utils.default_timeuid(data)
            # Ensure the active status on the new Configuration
            data['active'] = True
            try:
                jsonschema.validate(data,
                                    utils.schemas['configuration'])
            except (ValidationError, SchemaError):
                raise utils._compose_err_msg(400,
                                             "Invalid schema on document(s)", data)
            uids.append(data['uid'])
            database.configuration.insert_one(data)
        else:
            raise utils._compose_err_msg(500,
                                         status='ConfigurationHandler expects list or dict') 
        self.finish(ujson.dumps(uids))

    @gen.coroutine
    def put(self):
        database = self.settings['db']
        incoming = ujson.loads(self.request.body)
        try:
            query = incoming.pop('query')
            update = incoming.pop('update')
        except KeyError:
            raise utils._compose_err_msg(500,
                                         status='filter and update are both required fields')
        if any(x in update.keys() for x in ['uid', 'time']):
            raise utils._compose_err_msg(500,
                                   status='Time and uid cannot be updated')
        res = database.configuration.update_many(filter=query,
                                           update={'$set': update},
                                           upsert=False)
        self.finish(ujson.dumps(res.raw_result))

    @gen.coroutine
    def delete(self):
        database = self.settings['db']
        incoming = utils.unpack_params(self)
        #incoming = ujson.loads(self.request.body)
        try:
            uid_list = incoming.pop('uid_list')
        except KeyError:
            raise utils._compose_err_msg(500,
                                         status='delete require a list of uids')

        if not isinstance(uid_list, (list, tuple)):
            uid_list = [uid_list]

        res = database.configuration.update_many(filter={'uid': {'$in': uid_list}},
                                                 update={'$set': {'active': False}},
                                                 upsert=False)
        self.finish(ujson.dumps(res.raw_result))


class SchemaHandler(DefaultHandler):
    """Provides the json used for schema validation provided collection name"""
    @gen.coroutine
    def get(self):
        col = utils.unpack_params(self)
        self.write(utils.schemas[col])
        self.finish()

    @gen.coroutine
    def put(self):
        raise utils._compose_err_msg(405,
                                     status='Not allowed on server')

    @gen.coroutine
    def post(self):
        raise utils._compose_err_msg(405,
                                     status='Not allowed on server')



from __future__ import absolute_import
import argparse
import tornado.web
import tornado.ioloop
import tornado.options
import tornado.httpserver
from .server.engine import (ConfigurationReferenceHandler, SchemaHandler,
                            db_connect)
from .server.conf import load_configuration


class Application(tornado.web.Application):
    def __init__(self, db):
        handlers = [(r'/configuration', ConfigurationReferenceHandler),
                    (r'/schema', SchemaHandler)]
        settings = {'db': db}

        tornado.web.Application.__init__(self, handlers, **settings)


def parse_configuration(config=None):
    if not config:
        config = {k: v for k, v in
                  load_configuration('conftrak', 'CFTRK',
                                     ['mongo_host', 'mongo_port', 'timezone',
                                      'database', 'service_port'],
                                     allow_missing=True).items()
                  if v is not None}
    parser = argparse.ArgumentParser()
    parser.add_argument('--database', dest='database', type=str,
                        help='name of database to use')
    parser.add_argument('--mongo-host',
                        dest='mongo_host', type=str,
                        help='host to use')
    parser.add_argument('--timezone', dest='timezone', type=str,
                        help='Local timezone')
    parser.add_argument('--mongo-port', dest='mongo_port', type=int,
                        help='port to use to talk to mongo')
    parser.add_argument('--service-port', dest='service_port', type=int,
                        help='port listen to for clients')
    parser.add_argument('--log_file_prefix', dest='log_file_prefix', type=str,
                        help='Log file name that tornado logs are dumped')
    args = parser.parse_known_args()[0]
    if args.database is not None:
        config['database'] = args.database
    if args.mongo_host is not None:
        config['mongo_host'] = args.mongo_host
    if args.timezone is not None:
        config['timezone'] = args.timezone
    if args.mongo_port is not None:
        config['mongo_port'] = args.mongo_port
    service_port = args.service_port
    if service_port is None:
        service_port = 7771

    config['service_port'] = service_port
    config['log_file_prefix'] = args.log_file_prefix
    return config


def start_server(args=None):
    """
    ConfTrak service startup script.
    Returns tornado event loop provided configuration.

    Parameters
    ----------
    config: dict
        Command line arguments always have priority over local config or yaml
        files. Using these parameters, a tornado event loop is created. Keep
        in mind that this server is started in lazy fashion. It does not verify
        the existence of a mongo instance running on the specified location.
    """
    global server
    config = parse_configuration(args)
    db = db_connect(config['database'],
                    config['mongo_host'],
                    config['mongo_port'])

    tornado.options.parse_command_line({'log_file_prefix':
                                        config['log_file_prefix']})
    app = Application(db)
    print('Starting ConfTrak service with configuration ', config)
    server = tornado.httpserver.HTTPServer(app)
    server.listen(config['service_port'])

    tornado.ioloop.IOLoop.current().start()



# no yaml, just python or command line for now
import os
import yaml
import logging

logger = logging.getLogger(__name__)


def load_configuration(name, prefix, fields, allow_missing=False):
    """
    Load configuration data form a cascading series of locations.
    The precedence order is (highest priority last):
    1. CONDA_ENV/etc/{name}.yaml (if CONDA_ETC_ env is defined)
    2. /etc/{name}.yml
    3. ~/.config/{name}/connection.yml
    4. reading {PREFIX}_{FIELD} environmental variables
    Parameters
    ----------
    name : str
        The expected base-name of the configuration files
    prefix : str
        The prefix when looking for environmental variables
    fields : iterable of strings
        The required configuration fields
    Returns
    ------
    conf : dict
        Dictionary keyed on ``fields`` with the values extracted
    """
    filenames = [os.path.join('/etc', name + '.yml'),
                 os.path.join(os.path.expanduser('~'), '.config',
                              name, 'connection.yml'),
                ]
    if 'CONDA_ETC_' in os.environ:
        filenames.insert(0, os.path.join(os.environ['CONDA_ETC_'],
                                         name + '.yml'))

    config = {}
    for filename in filenames:
        if os.path.isfile(filename):
            with open(filename) as f:
                config.update(yaml.load(f))
            logger.debug("Using db connection specified in config file. \n%r",
                         config)

    for field in fields:
        var_name = prefix + '_' + field.upper().replace(' ', '_')
        config[field] = os.environ.get(var_name, config.get(field, None))

    missing = [k for k, v in config.items() if v is None]
    if not allow_missing and missing:
        raise KeyError("The configuration field(s) {0} were not found in any "
                       "file or environmental variable.".format(missing))
    return config


import uuid
import requests
import uuid
import time as ttime
from subprocess import Popen, STDOUT
import os
from pymongo import MongoClient

testing_config = dict(mongohost='localhost', mongoport=27017,
                      database='conftrak_test'+str(uuid.uuid4()), serviceport=7771,
                      tzone='US/Eastern')

try:
    from subprocess import DEVNULL # py3k only
except ImportError:
    import os
    DEVNULL = open(os.devnull, 'wb')

def conftrak_setup():
    global proc
    env = os.environ.copy()
    f = os.path.dirname(os.path.realpath(__file__))
    proc = Popen([os.path.join(os.path.split(env["_"])[0], "python"),
                  "../../startup.py", "--mongo-host",
                  testing_config["mongohost"], "--mongo-port",
                  str(testing_config['mongoport']), "--database",
                  testing_config['database'], "--timezone", testing_config['tzone'],
                  "--service-port", str(testing_config['serviceport'])]
                 , cwd=f, env=env, stdout=DEVNULL, stderr=STDOUT)
    print('Started the server with configuration..:{}'.format(testing_config))
    ttime.sleep(4) # make sure the process is started


def conftrak_teardown():
    proc2 = Popen(['kill', '-9', str(proc.pid)])
    ttime.sleep(5) # make sure the process is killed
    conn = MongoClient(host=testing_config['mongohost'],
                       port=testing_config['mongoport'])
    conn.drop_database(testing_config['database'])
    conn.close()
    ttime.sleep(2)




__author__ = 'hhslepicka'
import setuptools
import versioneer
import os
import sys

# NOTE: This file must remain Python 2 compatible for the foreseeable future,
# to ensure that we error out properly for people with outdated setuptools
# and/or pip.
min_version = (3, 6)
if sys.version_info < min_version:
    error = """
mxtools does not support Python {0}.{1}.
Python {2}.{3} and above is required. Check your Python version like so:

python3 --version

This may be due to an out-of-date pip. Make sure you have pip >= 9.0.1.
Upgrade pip like so:

pip install --upgrade pip
""".format(
        *(sys.version_info[:2] + min_version)
    )
    sys.exit(error)

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, "README.md"), encoding="utf-8") as readme_file:
    readme = readme_file.read()

with open(os.path.join(here, "requirements.txt")) as requirements_file:
    # Parse requirements.txt, ignoring any commented-out lines.
    requirements = [line for line in requirements_file.read().splitlines() if not line.startswith("#")]

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setuptools.setup(
    name='conftrak',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="Beamline and General Configurations Tracker and Storage",
    long_description=readme,
    install_requires=requirements,
    license="BSD 3-Clause",
    url="https://github.com/hhslepicka/conftrak.git",
    python_requires=">={}".format(".".join(str(n) for n in min_version)),
    packages=setuptools.find_packages(),
    package_data={'conftrak': ['schemas/*.json']},
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
    ],
)
